;=========================================================;
;                                                         ;
; Text Functions                                          ;
;                                                         ;
;---------------------------------------------------------;
; text.inc is an independent file, and doesn't require    ;
; any external functions or resources.  All data and      ;
; workings are all kept within the file.                  ;
; Most of the functions are independent and will work on  ;
; there own, with the exception of one or two which use   ;
; carrage return and things.                              ;
;                                                         ;
; Text mode functions for uses in programs.               ;                          
;=========================================================;

[BITS 32]

align 4


 ;----------------------------------------------------;
 ; Data.                                              ;
 ;----------------------------------------------------;


textcolor         db 0x09
screen_x          db 0
screen_y          db 0

align 4

 ;----------------------------------------------------;
 ; Sets the text pos                                  ;
 ;----------------------------------------------------;

set_text_pos:
	mov   [screen_x],al
	mov   [screen_y],ah
	ret

 ;----------------------------------------------------;
 ; TextColor                        ; Sets text color ;
 ;----------------------------------------------------;

TextColor:
        mov   byte[textcolor],al
        ret


 ;----------------------------------------------------;
 ; Inc screen pointer                                 ;
 ;----------------------------------------------------;

inc_scr_pointer:
	push  eax
	cld
	mov   al,[screen_x]
	cmp   al,79                  
	jb    no_incy
	call  carriage_return
	jmp   dend
no_incy:
	inc   al
	mov   [screen_x],al
dend:	pop   eax
	ret

 ;----------------------------------------------------;
 ; Carriage return                       ; next line  ;
 ;----------------------------------------------------;

carriage_return:
	push  eax
	mov byte  [screen_x],0
	mov   ah,[screen_y]     
	cmp   ah,49                  
	jb    no_scroll
	call  scroll
	jmp   cend
no_scroll:
	inc   ah    
	mov   [screen_y],ah     
cend:	pop   eax
	ret


 ;----------------------------------------------------;
 ; Gets screen pointer                                ;
 ;----------------------------------------------------;

get_scr_pointer:                                     
	push  eax                                          
	push  ebx
    xor   ebx,ebx

	mov   bl,[screen_x]
	shl   ebx,1
	add   edi, ebx
	mov   bl,[screen_y]
	mov   eax,0xa0
	mul   bx
	add   edi,eax
    pop   ebx
	pop   eax
       
	ret


 ;'''''''''''''''''''''''''''''''''''''''''''''''''''';
 ; Prints char               ; print's a char from al ;
 ;----------------------------------------------------;
 ;                                                    ;
 ;  Input:                                            ;
 ;      Al Ascii character to write.                  ;
 ;                                                    ;
 ; Output:                                            ;
 ;      None.                                         ;
 ;....................................................;

print_char:
	pushad                                   
	push  es
	push  gs
    cli
    push   ax
    mov   ax,18h  ; linear memory area selector
	mov   gs,ax                          
    pop   ax	
    mov   edi,0xb8000
	call  get_scr_pointer
    mov   ah,[textcolor]
	mov   [gs:edi],ax                    
	call  inc_scr_pointer
    sti
    pop   gs
	pop   es
    popad
    ret

 ;'''''''''''''''''''''''''''''''''''''''''''''''''''';
 ; Prints string      ; does what it say's on the box ;
 ;----------------------------------------------------;
 ;                                                    ;
 ;  Input:                                            ;
 ;      es:esi points to asciiz string to write.      ;
 ;                                                    ;
 ; Output:                                            ;
 ;      None.                                         ;
 ;....................................................;

print_string:
    pushad                                   
	push  es
	push  ds
aloop:
    mov   al,[ds:esi]
    cmp   al,0				           ; 0x00 = end of string
	je    gend
    cmp   al,0xd2                      ; 0xd2 = next byte is color byte
    je    ColorChange
	cmp   al,0x0d                      ; 0x0d = CR ( = \n )
	jne   no_cr
	call  carriage_return
	jmp   a1
ColorChange:
    inc   esi
    mov   al,[esi]
    mov   byte[textcolor],al
    inc   esi
    jmp   aloop
no_cr:
    ;mov   eax, esi
    ;mov   al, [esi]
	call  print_char
a1:	inc   esi
    jmp   aloop
gend:
	pop   ds
	pop   es
    popad
	ret


 ;----------------------------------------------------;
 ; write hex 32        ;prints a hex number from eax. ;
 ;----------------------------------------------------;

write_hex32:
	pushad
	mov   ebx,eax
loop1:
	rol   eax,8		                           
	call  write_hex16
	cmp   eax,ebx		                           
	jne   loop1
	popad
	ret

 ;----------------------------------------------------;
 ; write hex 16        ;prints a hex number from  al. ;
 ;----------------------------------------------------;

write_hex16:
	push  eax
	push  eax
	shr   al,4		                           
	call  hexget		                          
	call  print_char
	pop   eax
	call  hexget		                           
	call  print_char
	pop   eax
	ret


 ;----------------------------------------------------;
 ; Scroll                      ; scroll's screen up 1 ;
 ;----------------------------------------------------;
 ;                                                    ;
 ;   Input:                                           ;
 ;          none.                                     ;
 ;  Output:                                           ;
 ;          none.                                     ;
 ;....................................................;

scroll:
    pushad
    push  gs
    push  es
    push  ds
    mov   ax,0x18  ; select linear memory mapping
	mov   gs,ax  
    mov   al,0
	mov   ah,49           
    call  set_text_pos
	mov   edi,0xb8000
    add   edi,80*2*4 
    mov   esi,0xb8000
    add   esi,80*2*5 
	mov   ecx,80*46*2/4 
    cli
mloop:	
    mov   eax,[gs:esi]
	add   esi,4
	mov   [gs:edi],eax			
	add   edi,4
	dec   ecx
	jnz   mloop
    sti
    pop   ds
    pop   es
    pop   gs
	popad
	ret

 ;----------------------------------------------------;
 ; cls_text                     ; clear's full screen ;
 ;----------------------------------------------------;
 ;                                                    ;
 ;   Input:                                           ;
 ;          none.                                     ;
 ;  Output:                                           ;
 ;          none.                                     ;
 ;....................................................;

cls_text:
	push  eax
	push  ecx
	push  esi
	push  edi
    push  gs
    mov   ax,0x18    ; select the linear memory mapping
	mov   gs,ax
    cli
	mov   ecx,80*50     
	mov   edi,0B8000h
    mov   al," "		
	mov   bl,0Fh		
cls_loop:
	mov   byte [gs:edi],al
	inc   edi
	mov   byte [gs:edi],bl
	inc   edi
	dec   ecx
	jnz   cls_loop
	mov   [screen_x],byte 0
	mov   [screen_y],byte 4         
        sti
        pop   gs
	pop   edi
	pop   esi
	pop   ecx
	pop   eax
	ret

 ;----------------------------------------------------;
 ; hexget         convers from ASCII to hexadecimal.  ;
 ;----------------------------------------------------;

hexget:
	and   eax,0x0000000f
	or    eax,0x00000030
	cmp   eax,0x39
	ja    add7
	ret
add7:	add   eax,7		
	ret

 ;----------------------------------------------------;
 ; UpperCase.       ; converts a string to uppercase. ;
 ;----------------------------------------------------;
 ;                                                    ;
 ; es:edi = string buffer (0 termanated)              ;
 ;----------------------------------------------------;

UpperCase:
        pushad
        push  es
        mov   ax,0x10                               
        mov   es,ax    
UcaseNextChar:
        mov   al,byte[es:edi]
        cmp   al,0
        je    UcaseDone
        cmp   al,0x61
        jb    DontUcaseChar
        cmp   al,0x7a
        ja    DontUcaseChar
        sub   al,0x20
        mov   byte[es:edi],al  
DontUcaseChar:
        inc   edi
        jmp   UcaseNextChar
UcaseDone:
        pop   es
        popad
        ret
;------------------------------------------------------
